from typing import Union, cast
from httpx import Client
from .base import BaseHttp


class BlockingHttp(BaseHttp):
    _client: Client

    def _get(self, url, read_bytes: bool = False,
             read_json: bool = True, *args, **kwargs) -> Union[dict, bytes]:
        """Wrapped HTTPX Delete.
        """

        return cast(
            Union[dict, bytes],
            self.handle_resp(
                self._client.get(url, *args, **kwargs),
                json=read_json,
                read=read_bytes
            )
        )

    def _delete(self, url, *args, **kwargs) -> bool:
        """Wrapped HTTPX Delete.
        """

        return cast(
            bool,
            self.handle_resp(
                self._client.delete(url, *args, **kwargs),
                False
            )
        )

    def _post(self, url, read_json: bool = False,
              *args, **kwargs) -> Union[dict, bool]:
        """Wrapped HTTPX Post.
        """

        return cast(
            Union[dict, bool],
            self.handle_resp(
                self._client.post(url, *args, **kwargs),
                read_json
            )
        )

    def _put(self, url, *args, **kwargs) -> bool:
        """Wrapped HTTPX Put.
        """

        return cast(
            bool,
            self.handle_resp(
                self._client.put(url, *args, **kwargs),
                False
            )
        )



from typing import TYPE_CHECKING

from ..base import BackupBase
from ...routes import SERVER

if TYPE_CHECKING:
    from ... import Awaiting


class AwaitingBackup(BackupBase):
    _context: "Awaiting"

    async def restore(self) -> None:
        """Used to restore a backup.
        """

        await self._context._post(
            url=SERVER.backup_restore.format(self.server_id, self.backup_name),
        )



from typing import TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .. import Awaiting, Blocking


class MatchBase:
    def __init__(self, context: Union["Awaiting", "Blocking"],
                 match_id: str) -> None:
        self._context = context
        self.match_id = match_id



from typing import cast, TYPE_CHECKING

from .base import MatchBase
from ..routes import MATCHES
from ..models.match import MatchModel

if TYPE_CHECKING:
    from .. import Blocking


class BlockingMatch(MatchBase):
    _context: "Blocking"

    def get(self) -> MatchModel:
        """Gets details on a match

        Returns
        -------
        MatchModel
            Holds match details.
        """

        data = cast(
            dict,
            self._context._get(
                MATCHES.details.format(self.match_id)
            )
        )

        return MatchModel(data)
